import EventDispatcher from 'EventDispatcher';
import {ImageLoader} from 'jquery-image-loader';

class PureBox {
    /**
     *
     * @param {Object} options
     * @param {jQuery} $
     */
    constructor(options, $) {
        this.$ = $;
        this.$imgContainer = $(options['img_container']);
        this.options = options;
        this.imageLoader = new ImageLoader(5000, $);

        this.setup();
        this.initSetup();
    }

    setup() {
        var $ = this.$;

        var setupOptions = (options)=> {
            var defaultOptions = {
                auto_play: false,
                item: null,
                duration: 1000,
                play_duration: 3000,
                play_engine: null
            };

            this.options = $.extend(defaultOptions, options);
        };


        var setupItems = ()=> {
            // Add container and items to @parent.elems
            this.$items = this.options['item'] ? $(this.options['item']) : this.$imgContainer.find('a[href]');

            // Set an `index` for each item
            this.$items.each(function (index) {
                $(this).data('index', index);
            });
        };


        // setup all
        setupOptions(this.options);
        setupItems();
    }

    initSetup() {
        var setupPlayEngine = (PlayEngine)=> {
            if (!PlayEngine) {
                throw new Error('You must specify a PlayEngine.');
            }
            this.playEngine = new PlayEngine(this);
        };

        var setupEventHandlers = ()=> {
            this.$imgContainer.on('click', this.options['item'] || 'a[href]', (e)=> {
                e.preventDefault();
                var index = $(e.currentTarget).data('index');
                this.open(index);
                return false;
            });

            this.playEngine.UIElements.$prevButton.on('click', ()=> {
                this.prev();
            });
            this.playEngine.UIElements.$nextButton.on('click', ()=> {
                this.next();
            });
            this.playEngine.UIElements.$closeButton.on('click', ()=> {
                this.close();
            });

            var key = {
                Esc: 27,
                ArrowRight: 39,
                ArrowLeft: 37
            };

            $(window).on('keyup', (e)=> {
                switch (e.keyCode) {
                    case key.ArrowRight:
                        this.next();
                        break;
                    case key.ArrowLeft:
                        this.prev();
                        break;
                    case key.Esc:
                        this.close();
                }
            });
        };

        var setupEventDispatcher = ()=> {
            this.eventDispatcher = new EventDispatcher(['beforeNext', 'beforePrev', 'beforeOpen', 'beforeClose', 'afterNext', 'afterPrev', 'afterOpen', 'afterClose', 'prevImgLoaded', 'nextImgLoaded', 'openImgLoaded', 'beforeClick', 'afterClick', 'update']);
        };

        var setupFlags = ()=> {
            this.isPlaying = !!this.options.auto_play;
            this.currentIndex = 0;
            this.isOpeningClosing = false;
        };

        setupPlayEngine(this.options.play_engine);
        setupEventHandlers();
        setupEventDispatcher();
        setupFlags();
    }

    /**
     * Call this method to update internal status.
     * This method should be called after you inserted or removed items dynamically.
     * You can also give a options object to update options
     * (note that you can't update the `play_engine` option).
     *
     * @param options
     * @returns {PureBox}
     */
    update(options = {}) {
        this.options = this.$.extend(this.options, options);
        this.setup();
        this.eventDispatcher.trigger('update', [], this);
        return this;
    }

    /**
     * Move to the next slide.
     *
     * @returns {PureBox}
     */
    next() {
        if (this.isPlaying) {
            return this
        }
        this.currentIndex++;
        var isLastIndex = this.currentIndex > this.$items.length - 1;
        if (isLastIndex) {
            this.currentIndex = 0;
        }

        this.goto(this.currentIndex, 'Next');
        return this;
    }

    /**
     * Move to the previous slide.
     *
     * @returns {PureBox}
     */
    prev() {
        if (this.isPlaying) {
            return this;
        }

        this.currentIndex--;
        var isFirstIndex = this.currentIndex < 0;
        if (isFirstIndex) {
            this.currentIndex = this.$items.length - 1;
        }
        this.goto(this.currentIndex, 'Prev');
        return this;
    }

    /**
     * Go to a slide by it's index.
     * The concrete processing will be delegated to the specific Play-Engine.
     *
     * @param index
     * @param from
     * @returns {PureBox}
     */
    goto(index, from) {
        if (this.isPlaying) {
            return this;
        }

        this.isPlaying = true;
        var src     = this.getImgSrc(index),
            promise = this.handleImgLoading(src),
            args    = {
                src: src,
                index: index,
                from: from,
                preloadImg: promise
            };

        promise.done(()=> {
            this.eventDispatcher.trigger(`${from.toLowerCase()}ImgLoaded`, [args], this);
            this.eventDispatcher.trigger(`before${from}`, [args], this);
            this.playEngine.goto(args).then(()=> {
                this.isPlaying = false;
                this.eventDispatcher.trigger(`after${from}`, [args], this);
            });
        });

        return this;
    }

    /**
     * Play the slide show.
     *
     * @returns {PureBox}
     */
    play() {
        if (this.isAutoPlaying) {
            return this;
        }
        this.isAutoPlaying = true;
        this.playId = setInterval(()=> {
            this.next();
        }, this.options.play_duration);

        return this;
    }

    /**
     * Pause the slide show.
     *
     * @returns {PureBox}
     */
    pause() {
        this.isAutoPlaying = false;
        clearInterval(this.playId);
        
        return this;
    }

    /**
     * Open a item.
     * The concrete processing will be delegated to the specific Play-Engine.
     *
     * @param index
     * @returns {PureBox}
     */
    open(index) {
        if (this.isOpeningClosing) {
            return this;
        }
        this.currentIndex = index;
        var src     = this.getImgSrc(this.currentIndex),
            promise = this.handleImgLoading(src),
            args    = {index: this.currentIndex, src: src, preloadImg: promise};

        this.eventDispatcher.trigger('beforeOpen', [args], this);

        promise.done(()=> {
            this.eventDispatcher.trigger("openImgLoaded", [args], this);
        });

        this.playEngine.open(args).then(()=> {
            this.isOpeningClosing = false;
            this.eventDispatcher.trigger('afterOpen', [args], this);
        });

        return this;
    }

    /**
     * Close the slide show.
     * The concrete processing will be delegated to the specific Play-Engine.
     *
     * @returns {PureBox}
     */
    close() {
        if (this.isOpeningClosing) {
            return this;
        }
        var args = [{index: this.currentIndex}];
        this.eventDispatcher.trigger('beforeClose', args, this);

        this.playEngine.close(args).then(()=> {
            this.isOpeningClosing = false;
            this.eventDispatcher.trigger('afterClose', args, this);
        });

        return this;
    }

    getImgSrc(index) {
        return this.$items.eq(index).attr('href');
    }

    handleImgLoading(src) {
        // Show loader
        this.playEngine.UIElements.$loader.show();

        // After image is loaded hide loader
        return this.loadImgs(src).done(()=> {
            this.playEngine.UIElements.$loader.hide();
        });
    }

    loadImgs(imgs) {
        return this.imageLoader.load(imgs)
    }

    on() {
        this.eventDispatcher.on.apply(this.eventDispatcher, arguments);
        return this;
    }

    off() {
        this.eventDispatcher.off.apply(this.eventDispatcher, arguments);
        return this;
    }

    getImgContainer() {
        return this.$imgContainer;
    }

    getOptions() {
        return this.options;
    }
}

/*
 <<<GLOBAL
 window.PureBox = PureBox;
 GLOBAL;
 */

export default PureBox;
